@@ -61,7 +61,7 @@ impl<T: Deref<Target = Map>, K: Pod, V: Pod> HashMap<T, K, V> {
     }
 
     /// Returns a copy of the value associated with the key.
-    pub unsafe fn get(&self, key: &K, flags: u64) -> Result<V, MapError> {
+    pub fn get(&self, key: &K, flags: u64) -> Result<V, MapError> {
         let fd = self.inner.deref().fd_or_err()?;
         let value = bpf_map_lookup_elem(fd, key, flags).map_err(|(code, io_error)| {
             MapError::SyscallError {
@@ -75,13 +75,13 @@ impl<T: Deref<Target = Map>, K: Pod, V: Pod> HashMap<T, K, V> {
 
     /// An iterator visiting all key-value pairs in arbitrary order. The
     /// iterator item type is `Result<(K, V), MapError>`.
-    pub unsafe fn iter(&self) -> MapIter<'_, K, V> {
+    pub fn iter(&self) -> MapIter<'_, K, V, Self> {
         MapIter::new(self)
     }
 
     /// An iterator visiting all keys in arbitrary order. The iterator element
     /// type is `Result<K, MapError>`.
-    pub unsafe fn keys(&self) -> MapKeys<'_, K> {
+    pub fn keys(&self) -> MapKeys<'_, K> {
         MapKeys::new(&self.inner)
     }
 }
@@ -103,7 +103,7 @@ impl<T: Deref<Target = Map>, K: Pod, V: Pod> IterableMap<K, V> for HashMap<T, K,
         &self.inner
     }
 
-    unsafe fn get(&self, key: &K) -> Result<V, MapError> {
+    fn get(&self, key: &K) -> Result<V, MapError> {
         HashMap::get(self, key, 0)
     }
 }
@@ -158,9 +158,8 @@ mod tests {
 
     use super::*;
 
-    fn new_obj_map(name: &str) -> obj::Map {
+    fn new_obj_map() -> obj::Map {
         obj::Map {
-            name: name.to_string(),
             def: bpf_map_def {
                 map_type: BPF_MAP_TYPE_HASH as u32,
                 key_size: 4,
@@ -170,6 +169,7 @@ mod tests {
             },
             section_index: 0,
             data: Vec::new(),
+            kind: obj::MapKind::Other,
         }
     }
 
@@ -180,7 +180,7 @@ mod tests {
     #[test]
     fn test_wrong_key_size() {
         let map = Map {
-            obj: new_obj_map("TEST"),
+            obj: new_obj_map(),
             fd: None,
             pinned: false,
         };
@@ -196,7 +196,7 @@ mod tests {
     #[test]
     fn test_wrong_value_size() {
         let map = Map {
-            obj: new_obj_map("TEST"),
+            obj: new_obj_map(),
             fd: None,
             pinned: false,
         };
@@ -213,7 +213,6 @@ mod tests {
     fn test_try_from_wrong_map() {
         let map = Map {
             obj: obj::Map {
-                name: "TEST".to_string(),
                 def: bpf_map_def {
                     map_type: BPF_MAP_TYPE_PERF_EVENT_ARRAY as u32,
                     key_size: 4,
@@ -223,6 +222,7 @@ mod tests {
                 },
                 section_index: 0,
                 data: Vec::new(),
+                kind: obj::MapKind::Other,
             },
             fd: None,
             pinned: false,
@@ -237,7 +237,7 @@ mod tests {
     #[test]
     fn test_new_not_created() {
         let mut map = Map {
-            obj: new_obj_map("TEST"),
+            obj: new_obj_map(),
             fd: None,
             pinned: false,
         };
@@ -251,7 +251,7 @@ mod tests {
     #[test]
     fn test_new_ok() {
         let mut map = Map {
-            obj: new_obj_map("TEST"),
+            obj: new_obj_map(),
             fd: Some(42),
             pinned: false,
         };
@@ -262,7 +262,7 @@ mod tests {
     #[test]
     fn test_try_from_ok() {
         let map = Map {
-            obj: new_obj_map("TEST"),
+            obj: new_obj_map(),
             fd: Some(42),
             pinned: false,
         };
@@ -273,7 +273,6 @@ mod tests {
     fn test_try_from_ok_lru() {
         let map = Map {
             obj: obj::Map {
-                name: "TEST".to_string(),
                 def: bpf_map_def {
                     map_type: BPF_MAP_TYPE_LRU_HASH as u32,
                     key_size: 4,
@@ -283,6 +282,7 @@ mod tests {
                 },
                 section_index: 0,
                 data: Vec::new(),
+                kind: obj::MapKind::Other,
             },
             fd: Some(42),
             pinned: false,
@@ -296,7 +296,7 @@ mod tests {
         override_syscall(|_| sys_error(EFAULT));
 
         let mut map = Map {
-            obj: new_obj_map("TEST"),
+            obj: new_obj_map(),
             fd: Some(42),
             pinned: false,
         };
@@ -319,7 +319,7 @@ mod tests {
         });
 
         let mut map = Map {
-            obj: new_obj_map("TEST"),
+            obj: new_obj_map(),
             fd: Some(42),
             pinned: false,
         };
@@ -333,7 +333,7 @@ mod tests {
         override_syscall(|_| sys_error(EFAULT));
 
         let mut map = Map {
-            obj: new_obj_map("TEST"),
+            obj: new_obj_map(),
             fd: Some(42),
             pinned: false,
         };
@@ -356,7 +356,7 @@ mod tests {
         });
 
         let mut map = Map {
-            obj: new_obj_map("TEST"),
+            obj: new_obj_map(),
             fd: Some(42),
             pinned: false,
         };
@@ -369,14 +369,14 @@ mod tests {
     fn test_get_syscall_error() {
         override_syscall(|_| sys_error(EFAULT));
         let map = Map {
-            obj: new_obj_map("TEST"),
+            obj: new_obj_map(),
             fd: Some(42),
             pinned: false,
         };
         let hm = HashMap::<_, u32, u32>::new(&map).unwrap();
 
         assert!(matches!(
-            unsafe { hm.get(&1, 0) },
+            hm.get(&1, 0),
             Err(MapError::SyscallError { call, code: -1, io_error }) if call == "bpf_map_lookup_elem" && io_error.raw_os_error() == Some(EFAULT)
         ));
     }
@@ -391,16 +391,13 @@ mod tests {
             _ => sys_error(EFAULT),
         });
         let map = Map {
-            obj: new_obj_map("TEST"),
+            obj: new_obj_map(),
             fd: Some(42),
             pinned: false,
         };
         let hm = HashMap::<_, u32, u32>::new(&map).unwrap();
 
-        assert!(matches!(
-            unsafe { hm.get(&1, 0) },
-            Err(MapError::KeyNotFound)
-        ));
+        assert!(matches!(hm.get(&1, 0), Err(MapError::KeyNotFound)));
     }
 
     fn bpf_key<T: Copy>(attr: &bpf_attr) -> Option<T> {
@@ -430,12 +427,12 @@ mod tests {
             _ => sys_error(EFAULT),
         });
         let map = Map {
-            obj: new_obj_map("TEST"),
+            obj: new_obj_map(),
             fd: Some(42),
             pinned: false,
         };
         let hm = HashMap::<_, u32, u32>::new(&map).unwrap();
-        let keys = unsafe { hm.keys() }.collect::<Result<Vec<_>, _>>();
+        let keys = hm.keys().collect::<Result<Vec<_>, _>>();
         assert!(matches!(keys, Ok(ks) if ks.is_empty()))
     }
 
@@ -474,13 +471,13 @@ mod tests {
         });
 
         let map = Map {
-            obj: new_obj_map("TEST"),
+            obj: new_obj_map(),
             fd: Some(42),
             pinned: false,
         };
         let hm = HashMap::<_, u32, u32>::new(&map).unwrap();
 
-        let keys = unsafe { hm.keys() }.collect::<Result<Vec<_>, _>>().unwrap();
+        let keys = hm.keys().collect::<Result<Vec<_>, _>>().unwrap();
         assert_eq!(&keys, &[10, 20, 30])
     }
 
@@ -502,13 +499,13 @@ mod tests {
             _ => sys_error(EFAULT),
         });
         let map = Map {
-            obj: new_obj_map("TEST"),
+            obj: new_obj_map(),
             fd: Some(42),
             pinned: false,
         };
         let hm = HashMap::<_, u32, u32>::new(&map).unwrap();
 
-        let mut keys = unsafe { hm.keys() };
+        let mut keys = hm.keys();
         assert!(matches!(keys.next(), Some(Ok(10))));
         assert!(matches!(keys.next(), Some(Ok(20))));
         assert!(matches!(
@@ -532,12 +529,12 @@ mod tests {
             _ => sys_error(EFAULT),
         });
         let map = Map {
-            obj: new_obj_map("TEST"),
+            obj: new_obj_map(),
             fd: Some(42),
             pinned: false,
         };
         let hm = HashMap::<_, u32, u32>::new(&map).unwrap();
-        let items = unsafe { hm.iter() }.collect::<Result<Vec<_>, _>>().unwrap();
+        let items = hm.iter().collect::<Result<Vec<_>, _>>().unwrap();
         assert_eq!(&items, &[(10, 100), (20, 200), (30, 300)])
     }
 
@@ -565,13 +562,13 @@ mod tests {
             _ => sys_error(EFAULT),
         });
         let map = Map {
-            obj: new_obj_map("TEST"),
+            obj: new_obj_map(),
             fd: Some(42),
             pinned: false,
         };
         let hm = HashMap::<_, u32, u32>::new(&map).unwrap();
 
-        let items = unsafe { hm.iter() }.collect::<Result<Vec<_>, _>>().unwrap();
+        let items = hm.iter().collect::<Result<Vec<_>, _>>().unwrap();
         assert_eq!(&items, &[(10, 100), (30, 300)])
     }
 
@@ -599,13 +596,13 @@ mod tests {
             _ => sys_error(EFAULT),
         });
         let map = Map {
-            obj: new_obj_map("TEST"),
+            obj: new_obj_map(),
             fd: Some(42),
             pinned: false,
         };
         let hm = HashMap::<_, u32, u32>::new(&map).unwrap();
 
-        let mut iter = unsafe { hm.iter() };
+        let mut iter = hm.iter();
         assert!(matches!(iter.next(), Some(Ok((10, 100)))));
         assert!(matches!(iter.next(), Some(Ok((20, 200)))));
         assert!(matches!(
@@ -639,13 +636,13 @@ mod tests {
             _ => sys_error(EFAULT),
         });
         let map = Map {
-            obj: new_obj_map("TEST"),
+            obj: new_obj_map(),
             fd: Some(42),
             pinned: false,
         };
         let hm = HashMap::<_, u32, u32>::new(&map).unwrap();
 
-        let mut iter = unsafe { hm.iter() };
+        let mut iter = hm.iter();
         assert!(matches!(iter.next(), Some(Ok((10, 100)))));
         assert!(matches!(
             iter.next(),
