@@ -21,7 +21,7 @@
 //! use aya::programs::SkMsg;
 //!
 //! let intercept_egress = SockMap::try_from(bpf.map_mut("INTERCEPT_EGRESS")?)?;
-//! let prog: &mut SkMsg = bpf.program_mut("intercept_egress_packet")?.try_into()?;
+//! let prog: &mut SkMsg = bpf.program_mut("intercept_egress_packet").unwrap().try_into()?;
 //! prog.load()?;
 //! prog.attach(&intercept_egress)?;
 //! # Ok::<(), aya::BpfError>(())
@@ -50,6 +50,7 @@ mod map_lock;
 
 pub mod array;
 pub mod hash_map;
+pub mod lpm_trie;
 pub mod perf;
 pub mod queue;
 pub mod sock;
@@ -66,72 +67,142 @@ pub use stack::Stack;
 pub use stack_trace::StackTraceMap;
 
 #[derive(Error, Debug)]
+/// Errors occuring from working with Maps
 pub enum MapError {
+    /// Unable to find the map
     #[error("map `{name}` not found ")]
-    MapNotFound { name: String },
+    MapNotFound {
+        /// Map name
+        name: String,
+    },
 
+    /// Invalid map type encontered
     #[error("invalid map type {map_type}")]
-    InvalidMapType { map_type: u32 },
+    InvalidMapType {
+        /// The map type
+        map_type: u32,
+    },
 
+    /// Invalid map name encountered
     #[error("invalid map name `{name}`")]
-    InvalidName { name: String },
+    InvalidName {
+        /// The map name
+        name: String,
+    },
 
+    /// Pin path is invalid
     #[error("invalid map path `{error}`")]
-    InvalidPinPath { error: String },
+    InvalidPinPath {
+        /// The error message
+        error: String,
+    },
 
-    #[error("the map `{name}` has not been created")]
-    NotCreated { name: String },
+    /// The map has not been created
+    #[error("the map has not been created")]
+    NotCreated,
 
+    /// The map has already been created
     #[error("the map `{name}` has already been created")]
-    AlreadyCreated { name: String },
+    AlreadyCreated {
+        /// Map name
+        name: String,
+    },
 
+    /// The map has already been pinned
     #[error("the map `{name}` has already been pinned")]
-    AlreadyPinned { name: String },
+    AlreadyPinned {
+        /// Map name
+        name: String,
+    },
 
-    #[error("failed to create map `{name}`: {code}")]
+    /// Failed to create map
+    #[error("failed to create map `{name}` with code {code}")]
     CreateError {
+        /// Map name
         name: String,
+        /// Error code
         code: libc::c_long,
+        #[source]
+        /// Original io::Error
         io_error: io::Error,
     },
 
-    #[error("failed to pin map `{name}`: {code}")]
+    /// Failed to pin map
+    #[error("failed to pin map `{name}` with code {code}")]
     PinError {
+        /// Map Name
         name: String,
+        /// Error code
         code: libc::c_long,
+        #[source]
+        /// Original io::Error
         io_error: io::Error,
     },
 
+    /// Invalid key size
     #[error("invalid key size {size}, expected {expected}")]
-    InvalidKeySize { size: usize, expected: usize },
+    InvalidKeySize {
+        /// Size encountered
+        size: usize,
+        /// Size expected
+        expected: usize,
+    },
 
+    /// Invalid value size
     #[error("invalid value size {size}, expected {expected}")]
-    InvalidValueSize { size: usize, expected: usize },
+    InvalidValueSize {
+        /// Size encountered
+        size: usize,
+        /// Size expected
+        expected: usize,
+    },
 
+    /// Index is out of bounds
     #[error("the index is {index} but `max_entries` is {max_entries}")]
-    OutOfBounds { index: u32, max_entries: u32 },
+    OutOfBounds {
+        /// Index accessed
+        index: u32,
+        /// Map size
+        max_entries: u32,
+    },
 
+    /// Key not found
     #[error("key not found")]
     KeyNotFound,
 
+    /// Element not found
     #[error("element not found")]
     ElementNotFound,
 
+    /// Progam Not Loaded
     #[error("the program is not loaded")]
     ProgramNotLoaded,
 
-    #[error("the `{call}` syscall failed with code {code} io_error {io_error}")]
+    /// Syscall failed
+    #[error("the `{call}` syscall failed with code {code}")]
     SyscallError {
+        /// Syscall Name
         call: String,
+        /// Error code
         code: libc::c_long,
+        #[source]
+        /// Original io::Error
         io_error: io::Error,
     },
 
+    /// Map is borrowed mutably
     #[error("map `{name}` is borrowed mutably")]
-    BorrowError { name: String },
+    BorrowError {
+        /// Map name
+        name: String,
+    },
 
+    /// Map is already borrowed
     #[error("map `{name}` is already borrowed")]
-    BorrowMutError { name: String },
+    BorrowMutError {
+        /// Map name
+        name: String,
+    },
 }
 
 /// A generic handle to a BPF map.
@@ -141,22 +212,22 @@ pub enum MapError {
 pub struct Map {
     pub(crate) obj: obj::Map,
     pub(crate) fd: Option<RawFd>,
+    /// Indicates if this map has been pinned to bpffs
     pub pinned: bool,
 }
 
 impl Map {
-    pub fn create(&mut self) -> Result<RawFd, MapError> {
-        let name = self.obj.name.clone();
+    /// Creates a new map with the provided `name`
+    pub fn create(&mut self, name: &str) -> Result<RawFd, MapError> {
         if self.fd.is_some() {
-            return Err(MapError::AlreadyCreated { name });
+            return Err(MapError::AlreadyCreated { name: name.into() });
         }
 
-        let c_name =
-            CString::new(name.clone()).map_err(|_| MapError::InvalidName { name: name.clone() })?;
+        let c_name = CString::new(name).map_err(|_| MapError::InvalidName { name: name.into() })?;
 
         let fd = bpf_create_map(&c_name, &self.obj.def).map_err(|(code, io_error)| {
             MapError::CreateError {
-                name,
+                name: name.into(),
                 code,
                 io_error,
             }
@@ -167,12 +238,15 @@ impl Map {
         Ok(fd)
     }
 
-    pub(crate) fn from_pinned<P: AsRef<Path>>(&mut self, path: P) -> Result<RawFd, MapError> {
-        let name = self.obj.name.clone();
+    pub(crate) fn open_pinned<P: AsRef<Path>>(
+        &mut self,
+        name: &str,
+        path: P,
+    ) -> Result<RawFd, MapError> {
         if self.fd.is_some() {
-            return Err(MapError::AlreadyCreated { name });
+            return Err(MapError::AlreadyCreated { name: name.into() });
         }
-        let map_path = path.as_ref().join(self.name());
+        let map_path = path.as_ref().join(name);
         let path_string = match CString::new(map_path.to_str().unwrap()) {
             Ok(s) => s,
             Err(e) => {
@@ -182,7 +256,7 @@ impl Map {
             }
         };
         let fd = bpf_get_object(&path_string).map_err(|(code, io_error)| MapError::PinError {
-            name,
+            name: name.into(),
             code,
             io_error,
         })? as RawFd;
@@ -192,27 +266,20 @@ impl Map {
         Ok(fd)
     }
 
-    pub fn name(&self) -> &str {
-        &self.obj.name
-    }
-
+    /// Returns the [`bpf_map_type`] of this map
     pub fn map_type(&self) -> Result<bpf_map_type, MapError> {
         bpf_map_type::try_from(self.obj.def.map_type)
     }
 
     pub(crate) fn fd_or_err(&self) -> Result<RawFd, MapError> {
-        self.fd.ok_or_else(|| MapError::NotCreated {
-            name: self.obj.name.clone(),
-        })
+        self.fd.ok_or(MapError::NotCreated)
     }
 
-    pub(crate) fn pin<P: AsRef<Path>>(&mut self, path: P) -> Result<(), MapError> {
+    pub(crate) fn pin<P: AsRef<Path>>(&mut self, name: &str, path: P) -> Result<(), MapError> {
         if self.pinned {
-            return Err(MapError::AlreadyPinned {
-                name: self.name().to_string(),
-            });
+            return Err(MapError::AlreadyPinned { name: name.into() });
         }
-        let map_path = path.as_ref().join(self.name());
+        let map_path = path.as_ref().join(name);
         let fd = self.fd_or_err()?;
         let path_string = CString::new(map_path.to_string_lossy().into_owned()).map_err(|e| {
             MapError::InvalidPinPath {
@@ -229,10 +296,22 @@ impl Map {
     }
 }
 
-pub(crate) trait IterableMap<K: Pod, V> {
+impl Drop for Map {
+    fn drop(&mut self) {
+        // TODO: Replace this with an OwnedFd once that is stabilized.
+        if let Some(fd) = self.fd.take() {
+            unsafe { libc::close(fd) };
+        }
+    }
+}
+
+/// An iterable map
+pub trait IterableMap<K: Pod, V> {
+    /// Get a generic map handle
     fn map(&self) -> &Map;
 
-    unsafe fn get(&self, key: &K) -> Result<V, MapError>;
+    /// Get the value for the provided `key`
+    fn get(&self, key: &K) -> Result<V, MapError>;
 }
 
 /// Iterator returned by `map.keys()`.
@@ -290,14 +369,14 @@ impl<K: Pod> Iterator for MapKeys<'_, K> {
 }
 
 /// Iterator returned by `map.iter()`.
-pub struct MapIter<'coll, K: Pod, V> {
+pub struct MapIter<'coll, K: Pod, V, I: IterableMap<K, V>> {
     keys: MapKeys<'coll, K>,
-    map: &'coll dyn IterableMap<K, V>,
+    map: &'coll I,
     _v: PhantomData<V>,
 }
 
-impl<'coll, K: Pod, V> MapIter<'coll, K, V> {
-    fn new(map: &'coll dyn IterableMap<K, V>) -> MapIter<'coll, K, V> {
+impl<'coll, K: Pod, V, I: IterableMap<K, V>> MapIter<'coll, K, V, I> {
+    fn new(map: &'coll I) -> MapIter<'coll, K, V, I> {
         MapIter {
             keys: MapKeys::new(map.map()),
             map,
@@ -306,20 +385,17 @@ impl<'coll, K: Pod, V> MapIter<'coll, K, V> {
     }
 }
 
-impl<K: Pod, V> Iterator for MapIter<'_, K, V> {
+impl<K: Pod, V, I: IterableMap<K, V>> Iterator for MapIter<'_, K, V, I> {
     type Item = Result<(K, V), MapError>;
 
     fn next(&mut self) -> Option<Self::Item> {
         loop {
             match self.keys.next() {
-                Some(Ok(key)) => {
-                    let value = unsafe { self.map.get(&key) };
-                    match value {
-                        Ok(value) => return Some(Ok((key, value))),
-                        Err(MapError::KeyNotFound) => continue,
-                        Err(e) => return Some(Err(e)),
-                    }
-                }
+                Some(Ok(key)) => match self.map.get(&key) {
+                    Ok(value) => return Some(Ok((key, value))),
+                    Err(MapError::KeyNotFound) => continue,
+                    Err(e) => return Some(Err(e)),
+                },
                 Some(Err(e)) => return Some(Err(e)),
                 None => return None,
             }
@@ -475,14 +551,14 @@ mod tests {
     use crate::{
         bpf_map_def,
         generated::{bpf_cmd, bpf_map_type::BPF_MAP_TYPE_HASH},
+        obj::MapKind,
         sys::{override_syscall, Syscall},
     };
 
     use super::*;
 
-    fn new_obj_map(name: &str) -> obj::Map {
+    fn new_obj_map() -> obj::Map {
         obj::Map {
-            name: name.to_string(),
             def: bpf_map_def {
                 map_type: BPF_MAP_TYPE_HASH as u32,
                 key_size: 4,
@@ -492,12 +568,13 @@ mod tests {
             },
             section_index: 0,
             data: Vec::new(),
+            kind: MapKind::Other,
         }
     }
 
-    fn new_map(name: &str) -> Map {
+    fn new_map() -> Map {
         Map {
-            obj: new_obj_map(name),
+            obj: new_obj_map(),
             fd: None,
             pinned: false,
         }
@@ -513,18 +590,21 @@ mod tests {
             _ => Err((-1, io::Error::from_raw_os_error(EFAULT))),
         });
 
-        let mut map = new_map("foo");
-        assert!(matches!(map.create(), Ok(42)));
+        let mut map = new_map();
+        assert!(matches!(map.create("foo"), Ok(42)));
         assert_eq!(map.fd, Some(42));
-        assert!(matches!(map.create(), Err(MapError::AlreadyCreated { .. })));
+        assert!(matches!(
+            map.create("foo"),
+            Err(MapError::AlreadyCreated { .. })
+        ));
     }
 
     #[test]
     fn test_create_failed() {
         override_syscall(|_| Err((-42, io::Error::from_raw_os_error(EFAULT))));
 
-        let mut map = new_map("foo");
-        let ret = map.create();
+        let mut map = new_map();
+        let ret = map.create("foo");
         assert!(matches!(ret, Err(MapError::CreateError { .. })));
         if let Err(MapError::CreateError {
             name,
