@@ -1,8 +1,10 @@
 #[allow(clippy::module_inception)]
 mod btf;
+mod info;
 mod relocation;
 mod types;
 
 pub use btf::*;
+pub(crate) use info::*;
 pub use relocation::RelocationError;
 pub(crate) use types::*;
