@@ -2,6 +2,7 @@ use std::{
     collections::HashMap,
     convert::{TryFrom, TryInto},
     io, mem, ptr,
+    str::FromStr,
 };
 
 use thiserror::Error;
@@ -16,7 +17,7 @@ use crate::{
             fields_are_compatible, member_bit_field_size, member_bit_offset, types_are_compatible,
             BtfType, MAX_SPEC_LEN,
         },
-        Btf, BtfError, Object, Program,
+        Btf, BtfError, Object, Program, ProgramSection,
     },
     BpfError,
 };
@@ -84,18 +85,18 @@ fn err_type_name(name: &Option<String>) -> String {
 #[derive(Copy, Clone, Debug)]
 #[repr(u32)]
 enum RelocationKind {
-    FieldByteOffset = BPF_FIELD_BYTE_OFFSET,
-    FieldByteSize = BPF_FIELD_BYTE_SIZE,
-    FieldExists = BPF_FIELD_EXISTS,
-    FieldSigned = BPF_FIELD_SIGNED,
-    FieldLShift64 = BPF_FIELD_LSHIFT_U64,
-    FieldRShift64 = BPF_FIELD_RSHIFT_U64,
-    TypeIdLocal = BPF_TYPE_ID_LOCAL,
-    TypeIdTarget = BPF_TYPE_ID_TARGET,
-    TypeExists = BPF_TYPE_EXISTS,
-    TypeSize = BPF_TYPE_SIZE,
-    EnumVariantExists = BPF_ENUMVAL_EXISTS,
-    EnumVariantValue = BPF_ENUMVAL_VALUE,
+    FieldByteOffset = BPF_CORE_FIELD_BYTE_OFFSET,
+    FieldByteSize = BPF_CORE_FIELD_BYTE_SIZE,
+    FieldExists = BPF_CORE_FIELD_EXISTS,
+    FieldSigned = BPF_CORE_FIELD_SIGNED,
+    FieldLShift64 = BPF_CORE_FIELD_LSHIFT_U64,
+    FieldRShift64 = BPF_CORE_FIELD_RSHIFT_U64,
+    TypeIdLocal = BPF_CORE_TYPE_ID_LOCAL,
+    TypeIdTarget = BPF_CORE_TYPE_ID_TARGET,
+    TypeExists = BPF_CORE_TYPE_EXISTS,
+    TypeSize = BPF_CORE_TYPE_SIZE,
+    EnumVariantExists = BPF_CORE_ENUMVAL_EXISTS,
+    EnumVariantValue = BPF_CORE_ENUMVAL_VALUE,
 }
 
 impl TryFrom<u32> for RelocationKind {
@@ -105,18 +106,18 @@ impl TryFrom<u32> for RelocationKind {
         use RelocationKind::*;
 
         Ok(match v {
-            BPF_FIELD_BYTE_OFFSET => FieldByteOffset,
-            BPF_FIELD_BYTE_SIZE => FieldByteSize,
-            BPF_FIELD_EXISTS => FieldExists,
-            BPF_FIELD_SIGNED => FieldSigned,
-            BPF_FIELD_LSHIFT_U64 => FieldLShift64,
-            BPF_FIELD_RSHIFT_U64 => FieldRShift64,
-            BPF_TYPE_ID_LOCAL => TypeIdLocal,
-            BPF_TYPE_ID_TARGET => TypeIdTarget,
-            BPF_TYPE_EXISTS => TypeExists,
-            BPF_TYPE_SIZE => TypeSize,
-            BPF_ENUMVAL_EXISTS => EnumVariantExists,
-            BPF_ENUMVAL_VALUE => EnumVariantValue,
+            BPF_CORE_FIELD_BYTE_OFFSET => FieldByteOffset,
+            BPF_CORE_FIELD_BYTE_SIZE => FieldByteSize,
+            BPF_CORE_FIELD_EXISTS => FieldExists,
+            BPF_CORE_FIELD_SIGNED => FieldSigned,
+            BPF_CORE_FIELD_LSHIFT_U64 => FieldLShift64,
+            BPF_CORE_FIELD_RSHIFT_U64 => FieldRShift64,
+            BPF_CORE_TYPE_ID_LOCAL => TypeIdLocal,
+            BPF_CORE_TYPE_ID_TARGET => TypeIdTarget,
+            BPF_CORE_TYPE_EXISTS => TypeExists,
+            BPF_CORE_TYPE_SIZE => TypeSize,
+            BPF_CORE_ENUMVAL_EXISTS => EnumVariantExists,
+            BPF_CORE_ENUMVAL_VALUE => EnumVariantValue,
             kind => return Err(BtfError::InvalidRelocationKind { kind }),
         })
     }
@@ -161,12 +162,12 @@ impl Object {
         for (sec_name_off, relos) in btf_ext.relocations() {
             let section_name = local_btf.string_at(*sec_name_off)?;
 
-            // FIXME
-            let parts = section_name.split('/').collect::<Vec<_>>();
-            if parts.len() < 2 {
-                continue;
-            }
-            let section_name = parts[1];
+            let program_section = match ProgramSection::from_str(&section_name) {
+                Ok(program) => program,
+                Err(_) => continue,
+            };
+            let section_name = program_section.name();
+
             let program = self
                 .programs
                 .get_mut(section_name)
@@ -215,7 +216,7 @@ fn relocate_btf_program<'target>(
         let access_str = &*local_btf.string_at(rel.access_str_offset)?;
         let local_spec = AccessSpec::new(local_btf, rel.type_id, access_str, *rel)?;
 
-        let mut matches = match rel.kind {
+        let matches = match rel.kind {
             RelocationKind::TypeIdLocal => Vec::new(), // we don't need to look at target types to relocate this value
             _ => {
                 let candidates = match candidates_cache.get(&rel.type_id) {
@@ -243,7 +244,7 @@ fn relocate_btf_program<'target>(
         };
 
         let comp_rel = if !matches.is_empty() {
-            let mut matches = matches.drain(..);
+            let mut matches = matches.into_iter();
             let (_, target_spec, target_comp_rel) = matches.next().unwrap();
 
             // if there's more than one candidate, make sure that they all resolve to the
