@@ -3,7 +3,8 @@ mod relocation;
 
 use object::{
     read::{Object as ElfObject, ObjectSection, Section as ObjSection},
-    Endianness, ObjectSymbol, ObjectSymbolTable, RelocationTarget, SectionIndex, SymbolKind,
+    Endianness, ObjectSymbol, ObjectSymbolTable, RelocationTarget, SectionIndex, SectionKind,
+    SymbolKind,
 };
 use std::{
     collections::HashMap,
@@ -18,12 +19,14 @@ use relocation::*;
 
 use crate::{
     bpf_map_def,
-    generated::{bpf_insn, bpf_map_type::BPF_MAP_TYPE_ARRAY},
+    generated::{bpf_insn, bpf_map_type::BPF_MAP_TYPE_ARRAY, BPF_F_RDONLY_PROG},
     obj::btf::{Btf, BtfError, BtfExt},
     BpfError,
 };
 use std::slice::from_raw_parts_mut;
 
+use self::btf::{FuncSecInfo, LineSecInfo};
+
 const KERNEL_VERSION_ANY: u32 = 0xFFFF_FFFE;
 /// The first five __u32 of `bpf_map_def` must be defined.
 const MINIMUM_MAP_SIZE: usize = mem::size_of::<u32>() * 5;
@@ -40,14 +43,41 @@ pub struct Object {
     pub(crate) functions: HashMap<u64, Function>,
     pub(crate) relocations: HashMap<SectionIndex, HashMap<u64, Relocation>>,
     pub(crate) symbols_by_index: HashMap<usize, Symbol>,
+    pub(crate) section_sizes: HashMap<String, u64>,
+    // symbol_offset_by_name caches symbols that could be referenced from a
+    // BTF VAR type so the offsets can be fixed up
+    pub(crate) symbol_offset_by_name: HashMap<String, u64>,
+    pub(crate) text_section_index: Option<usize>,
+}
+
+#[derive(Debug, Clone, PartialEq)]
+pub(crate) enum MapKind {
+    Bss,
+    Data,
+    Rodata,
+    Other,
+}
+
+impl From<&str> for MapKind {
+    fn from(s: &str) -> Self {
+        if s == ".bss" {
+            MapKind::Bss
+        } else if s.starts_with(".data") {
+            MapKind::Data
+        } else if s.starts_with(".rodata") {
+            MapKind::Rodata
+        } else {
+            MapKind::Other
+        }
+    }
 }
 
 #[derive(Debug, Clone)]
 pub struct Map {
-    pub(crate) name: String,
     pub(crate) def: bpf_map_def,
     pub(crate) section_index: usize,
     pub(crate) data: Vec<u8>,
+    pub(crate) kind: MapKind,
 }
 
 #[derive(Debug, Clone)]
@@ -65,6 +95,10 @@ pub(crate) struct Function {
     pub(crate) section_index: SectionIndex,
     pub(crate) section_offset: usize,
     pub(crate) instructions: Vec<bpf_insn>,
+    pub(crate) func_info: FuncSecInfo,
+    pub(crate) line_info: LineSecInfo,
+    pub(crate) func_info_rec_size: usize,
+    pub(crate) line_info_rec_size: usize,
 }
 
 #[derive(Debug, Clone)]
@@ -81,10 +115,17 @@ pub enum ProgramSection {
     SkSkbStreamVerdict { name: String },
     SockOps { name: String },
     SchedClassifier { name: String },
+    CgroupSkb { name: String },
     CgroupSkbIngress { name: String },
     CgroupSkbEgress { name: String },
     LircMode2 { name: String },
     PerfEvent { name: String },
+    RawTracePoint { name: String },
+    Lsm { name: String },
+    BtfTracePoint { name: String },
+    FEntry { name: String },
+    FExit { name: String },
+    Extension { name: String },
 }
 
 impl ProgramSection {
@@ -102,10 +143,17 @@ impl ProgramSection {
             ProgramSection::SkSkbStreamVerdict { name } => name,
             ProgramSection::SockOps { name } => name,
             ProgramSection::SchedClassifier { name } => name,
+            ProgramSection::CgroupSkb { name } => name,
             ProgramSection::CgroupSkbIngress { name } => name,
             ProgramSection::CgroupSkbEgress { name } => name,
             ProgramSection::LircMode2 { name } => name,
             ProgramSection::PerfEvent { name } => name,
+            ProgramSection::RawTracePoint { name } => name,
+            ProgramSection::Lsm { name } => name,
+            ProgramSection::BtfTracePoint { name } => name,
+            ProgramSection::FEntry { name } => name,
+            ProgramSection::FExit { name } => name,
+            ProgramSection::Extension { name } => name,
         }
     }
 }
@@ -131,22 +179,55 @@ impl FromStr for ProgramSection {
             "uprobe" => UProbe { name },
             "uretprobe" => URetProbe { name },
             "xdp" => Xdp { name },
+            "tp_btf" => BtfTracePoint { name },
             _ if kind.starts_with("tracepoint") || kind.starts_with("tp") => {
                 // tracepoint sections are named `tracepoint/category/event_name`,
                 // and we want to parse the name as "category/event_name"
                 let name = section.splitn(2, '/').last().unwrap().to_owned();
                 TracePoint { name }
             }
-            "socket_filter" => SocketFilter { name },
+            "socket" => SocketFilter { name },
             "sk_msg" => SkMsg { name },
+            "sk_skb" => match &*name {
+                "stream_parser" => SkSkbStreamParser { name },
+                "stream_verdict" => SkSkbStreamVerdict { name },
+                _ => {
+                    return Err(ParseError::InvalidProgramSection {
+                        section: section.to_owned(),
+                    })
+                }
+            },
             "sk_skb/stream_parser" => SkSkbStreamParser { name },
             "sk_skb/stream_verdict" => SkSkbStreamVerdict { name },
             "sockops" => SockOps { name },
             "classifier" => SchedClassifier { name },
+            "cgroup_skb" => match &*name {
+                "ingress" => CgroupSkbIngress { name },
+                "egress" => CgroupSkbEgress { name },
+                _ => {
+                    return Err(ParseError::InvalidProgramSection {
+                        section: section.to_owned(),
+                    })
+                }
+            },
             "cgroup_skb/ingress" => CgroupSkbIngress { name },
             "cgroup_skb/egress" => CgroupSkbEgress { name },
+            "cgroup/skb" => CgroupSkb { name },
+            "cgroup" => match &*name {
+                "skb" => CgroupSkb { name },
+                _ => {
+                    return Err(ParseError::InvalidProgramSection {
+                        section: section.to_owned(),
+                    })
+                }
+            },
             "lirc_mode2" => LircMode2 { name },
             "perf_event" => PerfEvent { name },
+            "raw_tp" | "raw_tracepoint" => RawTracePoint { name },
+            "lsm" => Lsm { name },
+            "fentry" => FEntry { name },
+            "fexit" => FExit { name },
+            "freplace" => Extension { name },
             _ => {
                 return Err(ParseError::InvalidProgramSection {
                     section: section.to_owned(),
@@ -177,22 +258,45 @@ impl Object {
 
         if let Some(symbol_table) = obj.symbol_table() {
             for symbol in symbol_table.symbols() {
+                let name = symbol
+                    .name()
+                    .ok()
+                    .map(String::from)
+                    .ok_or(BtfError::InvalidSymbolName)?;
                 let sym = Symbol {
                     index: symbol.index().0,
-                    name: symbol.name().ok().map(String::from),
-                    section_index: symbol.section().index(),
+                    name: Some(name.clone()),
+                    section_index: symbol.section().index().map(|i| i.0),
                     address: symbol.address(),
                     size: symbol.size(),
                     is_definition: symbol.is_definition(),
-                    is_text: symbol.kind() == SymbolKind::Text,
+                    kind: symbol.kind(),
                 };
-                bpf_obj
-                    .symbols_by_index
-                    .insert(symbol.index().0, sym.clone());
+                bpf_obj.symbols_by_index.insert(symbol.index().0, sym);
+
+                if symbol.is_global() || symbol.kind() == SymbolKind::Data {
+                    bpf_obj.symbol_offset_by_name.insert(name, symbol.address());
+                }
+            }
+        }
+
+        // .BTF and .BTF.ext sections must be parsed first
+        // as they're required to prepare function and line information
+        // when parsing program sections
+        if let Some(s) = obj.section_by_name(".BTF") {
+            bpf_obj.parse_section(Section::try_from(&s)?)?;
+            if let Some(s) = obj.section_by_name(".BTF.ext") {
+                bpf_obj.parse_section(Section::try_from(&s)?)?;
             }
         }
 
         for s in obj.sections() {
+            if let Ok(name) = s.name() {
+                if name == ".BTF" || name == ".BTF.ext" {
+                    continue;
+                }
+            }
+
             bpf_obj.parse_section(Section::try_from(&s)?)?;
         }
 
@@ -211,9 +315,57 @@ impl Object {
             functions: HashMap::new(),
             relocations: HashMap::new(),
             symbols_by_index: HashMap::new(),
+            section_sizes: HashMap::new(),
+            symbol_offset_by_name: HashMap::new(),
+            text_section_index: None,
         }
     }
 
+    pub fn patch_map_data(&mut self, globals: HashMap<&str, &[u8]>) -> Result<(), ParseError> {
+        let symbols: HashMap<String, &Symbol> = self
+            .symbols_by_index
+            .iter()
+            .filter(|(_, s)| s.name.is_some())
+            .map(|(_, s)| (s.name.as_ref().unwrap().clone(), s))
+            .collect();
+
+        for (name, data) in globals {
+            if let Some(symbol) = symbols.get(name) {
+                if data.len() as u64 != symbol.size {
+                    return Err(ParseError::InvalidGlobalData {
+                        name: name.to_string(),
+                        sym_size: symbol.size,
+                        data_size: data.len(),
+                    });
+                }
+                let (_, map) = self
+                    .maps
+                    .iter_mut()
+                    // assumption: there is only one map created per section where we're trying to
+                    // patch data. this assumption holds true for the .rodata section at least
+                    .find(|(_, m)| symbol.section_index == Some(m.section_index))
+                    .ok_or_else(|| ParseError::MapNotFound {
+                        index: symbol.section_index.unwrap_or(0),
+                    })?;
+                let start = symbol.address as usize;
+                let end = start + symbol.size as usize;
+                if start > end || end > map.data.len() {
+                    return Err(ParseError::InvalidGlobalData {
+                        name: name.to_string(),
+                        sym_size: symbol.size,
+                        data_size: data.len(),
+                    });
+                }
+                map.data.splice(start..end, data.iter().cloned());
+            } else {
+                return Err(ParseError::SymbolNotFound {
+                    name: name.to_owned(),
+                });
+            }
+        }
+        Ok(())
+    }
+
     fn parse_btf(&mut self, section: &Section) -> Result<(), BtfError> {
         self.btf = Some(Btf::parse(section.data, self.endianness)?);
 
@@ -221,13 +373,32 @@ impl Object {
     }
 
     fn parse_btf_ext(&mut self, section: &Section) -> Result<(), BtfError> {
-        self.btf_ext = Some(BtfExt::parse(section.data, self.endianness)?);
+        self.btf_ext = Some(BtfExt::parse(
+            section.data,
+            self.endianness,
+            self.btf.as_ref().unwrap(),
+        )?);
         Ok(())
     }
 
     fn parse_program(&self, section: &Section) -> Result<Program, ParseError> {
         let prog_sec = ProgramSection::from_str(section.name)?;
         let name = prog_sec.name().to_owned();
+
+        let (func_info, line_info, func_info_rec_size, line_info_rec_size) =
+            if let Some(btf_ext) = &self.btf_ext {
+                let func_info = btf_ext.func_info.get(section.name);
+                let line_info = btf_ext.line_info.get(section.name);
+                (
+                    func_info,
+                    line_info,
+                    btf_ext.func_info_rec_size(),
+                    btf_ext.line_info_rec_size(),
+                )
+            } else {
+                (FuncSecInfo::default(), LineSecInfo::default(), 0, 0)
+            };
+
         Ok(Program {
             license: self.license.clone(),
             kernel_version: self.kernel_version,
@@ -238,15 +409,24 @@ impl Object {
                 section_index: section.index,
                 section_offset: 0,
                 instructions: copy_instructions(section.data)?,
+                func_info,
+                line_info,
+                func_info_rec_size,
+                line_info_rec_size,
             },
         })
     }
 
     fn parse_text_section(&mut self, mut section: Section) -> Result<(), ParseError> {
+        self.text_section_index = Some(section.index.0);
+
         let mut symbols_by_address = HashMap::new();
 
         for sym in self.symbols_by_index.values() {
-            if sym.is_definition && sym.is_text && sym.section_index == Some(section.index) {
+            if sym.is_definition
+                && sym.kind == SymbolKind::Text
+                && sym.section_index == Some(section.index.0)
+            {
                 if symbols_by_address.contains_key(&sym.address) {
                     return Err(ParseError::SymbolTableConflict {
                         section_index: section.index.0,
@@ -273,6 +453,38 @@ impl Object {
                 });
             }
 
+            let (func_info, line_info, func_info_rec_size, line_info_rec_size) =
+                if let Some(btf_ext) = &self.btf_ext {
+                    let bytes_offset = offset as u32 / INS_SIZE as u32;
+                    let section_size_bytes = sym.size as u32 / INS_SIZE as u32;
+
+                    let mut func_info = btf_ext.func_info.get(section.name);
+                    func_info.func_info = func_info
+                        .func_info
+                        .into_iter()
+                        .filter(|f| f.insn_off == bytes_offset)
+                        .collect();
+
+                    let mut line_info = btf_ext.line_info.get(section.name);
+                    line_info.line_info = line_info
+                        .line_info
+                        .into_iter()
+                        .filter(|l| {
+                            l.insn_off >= bytes_offset
+                                && l.insn_off < (bytes_offset + section_size_bytes) as u32
+                        })
+                        .collect();
+
+                    (
+                        func_info,
+                        line_info,
+                        btf_ext.func_info_rec_size(),
+                        btf_ext.line_info_rec_size(),
+                    )
+                } else {
+                    (FuncSecInfo::default(), LineSecInfo::default(), 0, 0)
+                };
+
             self.functions.insert(
                 sym.address,
                 Function {
@@ -283,6 +495,10 @@ impl Object {
                     instructions: copy_instructions(
                         &section.data[offset..offset + sym.size as usize],
                     )?,
+                    func_info,
+                    line_info,
+                    func_info_rec_size,
+                    line_info_rec_size,
                 },
             );
 
@@ -315,21 +531,22 @@ impl Object {
         {
             parts.push(parts[0]);
         }
-
-        match section.name {
-            name if name == ".bss" || name.starts_with(".data") || name.starts_with(".rodata") => {
+        self.section_sizes
+            .insert(section.name.to_owned(), section.size);
+        match section.kind {
+            BpfSectionKind::Data => {
+                self.maps
+                    .insert(section.name.to_string(), parse_map(&section, section.name)?);
+            }
+            BpfSectionKind::Text => self.parse_text_section(section)?,
+            BpfSectionKind::Btf => self.parse_btf(&section)?,
+            BpfSectionKind::BtfExt => self.parse_btf_ext(&section)?,
+            BpfSectionKind::Maps => {
+                let name = section.name.splitn(2, '/').last().unwrap();
                 self.maps
                     .insert(name.to_string(), parse_map(&section, name)?);
             }
-            name if name.starts_with(".text") => self.parse_text_section(section)?,
-            ".BTF" => self.parse_btf(&section)?,
-            ".BTF.ext" => self.parse_btf_ext(&section)?,
-            map if map.starts_with("maps/") => {
-                let name = map.splitn(2, '/').last().unwrap();
-                self.maps
-                    .insert(name.to_string(), parse_map(&section, name)?);
-            }
-            name if is_program_section(name) => {
+            BpfSectionKind::Program => {
                 let program = self.parse_program(&section)?;
                 self.programs
                     .insert(program.section.name().to_owned(), program);
@@ -344,8 +561,10 @@ impl Object {
                     );
                 }
             }
-
-            _ => {}
+            BpfSectionKind::Undefined
+            | BpfSectionKind::BtfMaps
+            | BpfSectionKind::License
+            | BpfSectionKind::Version => {}
         }
 
         Ok(())
@@ -393,14 +612,70 @@ pub enum ParseError {
 
     #[error("invalid symbol, index `{index}` name: {}", .name.as_ref().unwrap_or(&"[unknown]".into()))]
     InvalidSymbol { index: usize, name: Option<String> },
+
+    #[error("symbol {name} has size `{sym_size}`, but provided data is of size `{data_size}`")]
+    InvalidGlobalData {
+        name: String,
+        sym_size: u64,
+        data_size: usize,
+    },
+
+    #[error("symbol with name {name} not found in the symbols table")]
+    SymbolNotFound { name: String },
+
+    #[error("map for section with index {index} not found")]
+    MapNotFound { index: usize },
+}
+
+#[derive(Debug)]
+enum BpfSectionKind {
+    Undefined,
+    Maps,
+    BtfMaps,
+    Program,
+    Data,
+    Text,
+    Btf,
+    BtfExt,
+    License,
+    Version,
+}
+
+impl BpfSectionKind {
+    fn from_name(name: &str) -> BpfSectionKind {
+        if name.starts_with("license") {
+            BpfSectionKind::License
+        } else if name.starts_with("version") {
+            BpfSectionKind::Version
+        } else if name.starts_with("maps") {
+            BpfSectionKind::Maps
+        } else if name.starts_with(".maps") {
+            BpfSectionKind::BtfMaps
+        } else if name.starts_with(".text") {
+            BpfSectionKind::Text
+        } else if name.starts_with(".bss")
+            || name.starts_with(".data")
+            || name.starts_with(".rodata")
+        {
+            BpfSectionKind::Data
+        } else if name == ".BTF" {
+            BpfSectionKind::Btf
+        } else if name == ".BTF.ext" {
+            BpfSectionKind::BtfExt
+        } else {
+            BpfSectionKind::Undefined
+        }
+    }
 }
 
 #[derive(Debug)]
 struct Section<'a> {
     index: SectionIndex,
+    kind: BpfSectionKind,
     address: u64,
     name: &'a str,
     data: &'a [u8],
+    size: u64,
     relocations: Vec<Relocation>,
 }
 
@@ -413,12 +688,24 @@ impl<'data, 'file, 'a> TryFrom<&'a ObjSection<'data, 'file>> for Section<'a> {
             index: index.0,
             source,
         };
-
+        let name = section.name().map_err(map_err)?;
+        let kind = match BpfSectionKind::from_name(name) {
+            BpfSectionKind::Undefined => {
+                if section.kind() == SectionKind::Text && section.size() > 0 {
+                    BpfSectionKind::Program
+                } else {
+                    BpfSectionKind::Undefined
+                }
+            }
+            k => k,
+        };
         Ok(Section {
             index,
+            kind,
             address: section.address(),
-            name: section.name().map_err(map_err)?,
+            name,
             data: section.data().map_err(map_err)?,
+            size: section.size(),
             relocations: section
                 .relocations()
                 .map(|(offset, r)| {
@@ -491,26 +778,32 @@ impl From<KernelVersion> for u32 {
 }
 
 fn parse_map(section: &Section, name: &str) -> Result<Map, ParseError> {
-    let (def, data) = if name == ".bss" || name.starts_with(".data") || name.starts_with(".rodata")
-    {
-        let def = bpf_map_def {
-            map_type: BPF_MAP_TYPE_ARRAY as u32,
-            key_size: mem::size_of::<u32>() as u32,
-            value_size: section.data.len() as u32,
-            max_entries: 1,
-            map_flags: 0, /* FIXME: set rodata readonly */
-            ..Default::default()
-        };
-        (def, section.data.to_vec())
-    } else {
-        (parse_map_def(name, section.data)?, Vec::new())
+    let kind = MapKind::from(name);
+    let (def, data) = match kind {
+        MapKind::Bss | MapKind::Data | MapKind::Rodata => {
+            let def = bpf_map_def {
+                map_type: BPF_MAP_TYPE_ARRAY as u32,
+                key_size: mem::size_of::<u32>() as u32,
+                // We need to use section.size here since
+                // .bss will always have data.len() == 0
+                value_size: section.size as u32,
+                max_entries: 1,
+                map_flags: if kind == MapKind::Rodata {
+                    BPF_F_RDONLY_PROG
+                } else {
+                    0
+                },
+                ..Default::default()
+            };
+            (def, section.data.to_vec())
+        }
+        MapKind::Other => (parse_map_def(name, section.data)?, Vec::new()),
     };
-
     Ok(Map {
         section_index: section.index.0,
-        name: name.to_string(),
         def,
         data,
+        kind,
     })
 }
 
@@ -534,7 +827,7 @@ fn parse_map_def(name: &str, data: &[u8]) -> Result<bpf_map_def, ParseError> {
     }
 }
 
-fn copy_instructions(data: &[u8]) -> Result<Vec<bpf_insn>, ParseError> {
+pub(crate) fn copy_instructions(data: &[u8]) -> Result<Vec<bpf_insn>, ParseError> {
     if data.len() % mem::size_of::<bpf_insn>() > 0 {
         return Err(ParseError::InvalidProgramCode);
     }
@@ -550,50 +843,22 @@ fn copy_instructions(data: &[u8]) -> Result<Vec<bpf_insn>, ParseError> {
     Ok(instructions)
 }
 
-fn is_program_section(name: &str) -> bool {
-    for prefix in &[
-        "classifier",
-        "cgroup/skb",
-        "cgroup_skb/egress",
-        "cgroup_skb/ingress",
-        "kprobe",
-        "kretprobe",
-        "lirc_mode2",
-        "perf_event",
-        "sk_msg",
-        "sk_skb/stream_parser",
-        "sk_skb/stream_verdict",
-        "socket_filter",
-        "sockops",
-        "tp",
-        "tracepoint",
-        "uprobe",
-        "uretprobe",
-        "xdp",
-    ] {
-        if name.starts_with(prefix) {
-            return true;
-        }
-    }
-
-    false
-}
-
 #[cfg(test)]
 mod tests {
     use matches::assert_matches;
     use object::Endianness;
-    use std::slice;
 
     use super::*;
     use crate::PinningType;
 
-    fn fake_section<'a>(name: &'a str, data: &'a [u8]) -> Section<'a> {
+    fn fake_section<'a>(kind: BpfSectionKind, name: &'a str, data: &'a [u8]) -> Section<'a> {
         Section {
             index: SectionIndex(0),
+            kind,
             address: 0,
             name,
             data,
+            size: data.len() as u64,
             relocations: Vec::new(),
         }
     }
@@ -609,8 +874,8 @@ mod tests {
     }
 
     fn bytes_of<T>(val: &T) -> &[u8] {
-        let size = mem::size_of::<T>();
-        unsafe { slice::from_raw_parts(slice::from_ref(val).as_ptr().cast(), size) }
+        // Safety: This is for testing only
+        unsafe { crate::util::bytes_of(val) }
     }
 
     #[test]
@@ -733,7 +998,7 @@ mod tests {
     #[test]
     fn test_parse_map_error() {
         assert!(matches!(
-            parse_map(&fake_section("maps/foo", &[]), "foo"),
+            parse_map(&fake_section(BpfSectionKind::Maps, "maps/foo", &[]), "foo",),
             Err(ParseError::InvalidMapDefinition { .. })
         ));
     }
@@ -743,6 +1008,7 @@ mod tests {
         assert!(matches!(
             parse_map(
                 &fake_section(
+                    BpfSectionKind::Maps,
                     "maps/foo",
                     bytes_of(&bpf_map_def {
                         map_type: 1,
@@ -758,7 +1024,6 @@ mod tests {
             ),
             Ok(Map {
                 section_index: 0,
-                name,
                 def: bpf_map_def {
                     map_type: 1,
                     key_size: 2,
@@ -768,8 +1033,9 @@ mod tests {
                     id: 0,
                     pinning: PinningType::None,
                 },
-                data
-            }) if name == "foo" && data.is_empty()
+                data,
+                ..
+            }) if data.is_empty()
         ))
     }
 
@@ -779,6 +1045,7 @@ mod tests {
         assert!(matches!(
             parse_map(
                 &fake_section(
+                    BpfSectionKind::Data,
                     ".bss",
                     map_data,
                 ),
@@ -786,7 +1053,6 @@ mod tests {
             ),
             Ok(Map {
                 section_index: 0,
-                name,
                 def: bpf_map_def {
                     map_type: _map_type,
                     key_size: 4,
@@ -796,8 +1062,9 @@ mod tests {
                     id: 0,
                     pinning: PinningType::None,
                 },
-                data
-            }) if name == ".bss" && data == map_data && value_size == map_data.len() as u32
+                data,
+                kind
+            }) if data == map_data && value_size == map_data.len() as u32 && kind == MapKind::Bss
         ))
     }
 
@@ -814,7 +1081,11 @@ mod tests {
         let obj = fake_obj();
 
         assert_matches!(
-            obj.parse_program(&fake_section("kprobe/foo", &42u32.to_ne_bytes(),),),
+            obj.parse_program(&fake_section(
+                BpfSectionKind::Program,
+                "kprobe/foo",
+                &42u32.to_ne_bytes(),
+            )),
             Err(ParseError::InvalidProgramCode)
         );
     }
@@ -824,7 +1095,7 @@ mod tests {
         let obj = fake_obj();
 
         assert_matches!(
-            obj.parse_program(&fake_section("kprobe/foo", bytes_of(&fake_ins()))),
+            obj.parse_program(&fake_section(BpfSectionKind::Program,"kprobe/foo", bytes_of(&fake_ins()))),
             Ok(Program {
                 license,
                 kernel_version: KernelVersion::Any,
@@ -834,9 +1105,8 @@ mod tests {
                     address: 0,
                     section_index: SectionIndex(0),
                     section_offset: 0,
-                    instructions
-                }
-            }) if license.to_string_lossy() == "GPL" && name == "foo" && instructions.len() == 1
+                    instructions,
+                    ..} }) if license.to_string_lossy() == "GPL" && name == "foo" && instructions.len() == 1
         );
     }
 
@@ -846,6 +1116,7 @@ mod tests {
 
         assert_matches!(
             obj.parse_section(fake_section(
+                BpfSectionKind::Maps,
                 "maps/foo",
                 bytes_of(&bpf_map_def {
                     map_type: 1,
@@ -855,7 +1126,7 @@ mod tests {
                     map_flags: 5,
                     ..Default::default()
                 })
-            ),),
+            )),
             Ok(())
         );
         assert!(obj.maps.get("foo").is_some());
@@ -865,31 +1136,35 @@ mod tests {
     fn test_parse_section_data() {
         let mut obj = fake_obj();
         assert_matches!(
-            obj.parse_section(fake_section(".bss", b"map data"),),
+            obj.parse_section(fake_section(BpfSectionKind::Data, ".bss", b"map data")),
             Ok(())
         );
         assert!(obj.maps.get(".bss").is_some());
 
         assert_matches!(
-            obj.parse_section(fake_section(".rodata", b"map data"),),
+            obj.parse_section(fake_section(BpfSectionKind::Data, ".rodata", b"map data")),
             Ok(())
         );
         assert!(obj.maps.get(".rodata").is_some());
 
         assert_matches!(
-            obj.parse_section(fake_section(".rodata.boo", b"map data"),),
+            obj.parse_section(fake_section(
+                BpfSectionKind::Data,
+                ".rodata.boo",
+                b"map data"
+            )),
             Ok(())
         );
         assert!(obj.maps.get(".rodata.boo").is_some());
 
         assert_matches!(
-            obj.parse_section(fake_section(".data", b"map data"),),
+            obj.parse_section(fake_section(BpfSectionKind::Data, ".data", b"map data")),
             Ok(())
         );
         assert!(obj.maps.get(".data").is_some());
 
         assert_matches!(
-            obj.parse_section(fake_section(".data.boo", b"map data"),),
+            obj.parse_section(fake_section(BpfSectionKind::Data, ".data.boo", b"map data")),
             Ok(())
         );
         assert!(obj.maps.get(".data.boo").is_some());
@@ -900,7 +1175,11 @@ mod tests {
         let mut obj = fake_obj();
 
         assert_matches!(
-            obj.parse_section(fake_section("kprobe/foo", bytes_of(&fake_ins()))),
+            obj.parse_section(fake_section(
+                BpfSectionKind::Program,
+                "kprobe/foo",
+                bytes_of(&fake_ins())
+            )),
             Ok(())
         );
         assert_matches!(
@@ -917,7 +1196,11 @@ mod tests {
         let mut obj = fake_obj();
 
         assert_matches!(
-            obj.parse_section(fake_section("uprobe/foo", bytes_of(&fake_ins()))),
+            obj.parse_section(fake_section(
+                BpfSectionKind::Program,
+                "uprobe/foo",
+                bytes_of(&fake_ins())
+            )),
             Ok(())
         );
         assert_matches!(
@@ -934,7 +1217,11 @@ mod tests {
         let mut obj = fake_obj();
 
         assert_matches!(
-            obj.parse_section(fake_section("tracepoint/foo", bytes_of(&fake_ins()))),
+            obj.parse_section(fake_section(
+                BpfSectionKind::Program,
+                "tracepoint/foo",
+                bytes_of(&fake_ins())
+            )),
             Ok(())
         );
         assert_matches!(
@@ -946,7 +1233,11 @@ mod tests {
         );
 
         assert_matches!(
-            obj.parse_section(fake_section("tp/foo/bar", bytes_of(&fake_ins()))),
+            obj.parse_section(fake_section(
+                BpfSectionKind::Program,
+                "tp/foo/bar",
+                bytes_of(&fake_ins())
+            )),
             Ok(())
         );
         assert_matches!(
@@ -963,7 +1254,11 @@ mod tests {
         let mut obj = fake_obj();
 
         assert_matches!(
-            obj.parse_section(fake_section("socket_filter/foo", bytes_of(&fake_ins()))),
+            obj.parse_section(fake_section(
+                BpfSectionKind::Program,
+                "socket/foo",
+                bytes_of(&fake_ins())
+            )),
             Ok(())
         );
         assert_matches!(
@@ -980,7 +1275,11 @@ mod tests {
         let mut obj = fake_obj();
 
         assert_matches!(
-            obj.parse_section(fake_section("xdp/foo", bytes_of(&fake_ins()))),
+            obj.parse_section(fake_section(
+                BpfSectionKind::Program,
+                "xdp/foo",
+                bytes_of(&fake_ins())
+            )),
             Ok(())
         );
         assert_matches!(
@@ -991,4 +1290,292 @@ mod tests {
             })
         );
     }
+
+    #[test]
+    fn test_parse_section_raw_tp() {
+        let mut obj = fake_obj();
+
+        assert_matches!(
+            obj.parse_section(fake_section(
+                BpfSectionKind::Program,
+                "raw_tp/foo",
+                bytes_of(&fake_ins())
+            )),
+            Ok(())
+        );
+        assert_matches!(
+            obj.programs.get("foo"),
+            Some(Program {
+                section: ProgramSection::RawTracePoint { .. },
+                ..
+            })
+        );
+
+        assert_matches!(
+            obj.parse_section(fake_section(
+                BpfSectionKind::Program,
+                "raw_tracepoint/bar",
+                bytes_of(&fake_ins())
+            )),
+            Ok(())
+        );
+        assert_matches!(
+            obj.programs.get("bar"),
+            Some(Program {
+                section: ProgramSection::RawTracePoint { .. },
+                ..
+            })
+        );
+    }
+
+    #[test]
+    fn test_parse_section_lsm() {
+        let mut obj = fake_obj();
+
+        assert_matches!(
+            obj.parse_section(fake_section(
+                BpfSectionKind::Program,
+                "lsm/foo",
+                bytes_of(&fake_ins())
+            )),
+            Ok(())
+        );
+        assert_matches!(
+            obj.programs.get("foo"),
+            Some(Program {
+                section: ProgramSection::Lsm { .. },
+                ..
+            })
+        );
+    }
+
+    #[test]
+    fn test_parse_section_btf_tracepoint() {
+        let mut obj = fake_obj();
+
+        assert_matches!(
+            obj.parse_section(fake_section(
+                BpfSectionKind::Program,
+                "tp_btf/foo",
+                bytes_of(&fake_ins())
+            )),
+            Ok(())
+        );
+        assert_matches!(
+            obj.programs.get("foo"),
+            Some(Program {
+                section: ProgramSection::BtfTracePoint { .. },
+                ..
+            })
+        );
+    }
+
+    #[test]
+    fn test_parse_section_skskb_unnamed() {
+        let mut obj = fake_obj();
+
+        assert_matches!(
+            obj.parse_section(fake_section(
+                BpfSectionKind::Program,
+                "sk_skb/stream_parser",
+                bytes_of(&fake_ins())
+            )),
+            Ok(())
+        );
+        assert_matches!(
+            obj.programs.get("stream_parser"),
+            Some(Program {
+                section: ProgramSection::SkSkbStreamParser { .. },
+                ..
+            })
+        );
+    }
+
+    #[test]
+    fn test_parse_section_skskb_named() {
+        let mut obj = fake_obj();
+
+        assert_matches!(
+            obj.parse_section(fake_section(
+                BpfSectionKind::Program,
+                "sk_skb/stream_parser/my_parser",
+                bytes_of(&fake_ins())
+            )),
+            Ok(())
+        );
+        assert_matches!(
+            obj.programs.get("my_parser"),
+            Some(Program {
+                section: ProgramSection::SkSkbStreamParser { .. },
+                ..
+            })
+        );
+    }
+
+    #[test]
+    fn test_parse_section_fentry() {
+        let mut obj = fake_obj();
+
+        assert_matches!(
+            obj.parse_section(fake_section(
+                BpfSectionKind::Program,
+                "fentry/foo",
+                bytes_of(&fake_ins())
+            )),
+            Ok(())
+        );
+        assert_matches!(
+            obj.programs.get("foo"),
+            Some(Program {
+                section: ProgramSection::FEntry { .. },
+                ..
+            })
+        );
+    }
+
+    #[test]
+    fn test_parse_section_fexit() {
+        let mut obj = fake_obj();
+
+        assert_matches!(
+            obj.parse_section(fake_section(
+                BpfSectionKind::Program,
+                "fexit/foo",
+                bytes_of(&fake_ins())
+            )),
+            Ok(())
+        );
+        assert_matches!(
+            obj.programs.get("foo"),
+            Some(Program {
+                section: ProgramSection::FExit { .. },
+                ..
+            })
+        );
+    }
+
+    #[test]
+    fn test_parse_section_cgroup_skb_ingress_unnamed() {
+        let mut obj = fake_obj();
+
+        assert_matches!(
+            obj.parse_section(fake_section(
+                BpfSectionKind::Program,
+                "cgroup_skb/ingress",
+                bytes_of(&fake_ins())
+            )),
+            Ok(())
+        );
+        assert_matches!(
+            obj.programs.get("ingress"),
+            Some(Program {
+                section: ProgramSection::CgroupSkbIngress { .. },
+                ..
+            })
+        );
+    }
+
+    #[test]
+    fn test_parse_section_cgroup_skb_ingress_named() {
+        let mut obj = fake_obj();
+
+        assert_matches!(
+            obj.parse_section(fake_section(
+                BpfSectionKind::Program,
+                "cgroup_skb/ingress/foo",
+                bytes_of(&fake_ins())
+            )),
+            Ok(())
+        );
+        assert_matches!(
+            obj.programs.get("foo"),
+            Some(Program {
+                section: ProgramSection::CgroupSkbIngress { .. },
+                ..
+            })
+        );
+    }
+
+    #[test]
+    fn test_parse_section_cgroup_skb_no_direction_unamed() {
+        let mut obj = fake_obj();
+
+        assert_matches!(
+            obj.parse_section(fake_section(
+                BpfSectionKind::Program,
+                "cgroup/skb",
+                bytes_of(&fake_ins())
+            )),
+            Ok(())
+        );
+        assert_matches!(
+            obj.programs.get("skb"),
+            Some(Program {
+                section: ProgramSection::CgroupSkb { .. },
+                ..
+            })
+        );
+    }
+
+    #[test]
+    fn test_parse_section_cgroup_skb_no_direction_named() {
+        let mut obj = fake_obj();
+
+        assert_matches!(
+            obj.parse_section(fake_section(
+                BpfSectionKind::Program,
+                "cgroup/skb/foo",
+                bytes_of(&fake_ins())
+            )),
+            Ok(())
+        );
+        assert_matches!(
+            obj.programs.get("foo"),
+            Some(Program {
+                section: ProgramSection::CgroupSkb { .. },
+                ..
+            })
+        );
+    }
+
+    #[test]
+    fn test_patch_map_data() {
+        let mut obj = fake_obj();
+        obj.maps.insert(
+            ".rodata".to_string(),
+            Map {
+                def: bpf_map_def {
+                    map_type: BPF_MAP_TYPE_ARRAY as u32,
+                    key_size: mem::size_of::<u32>() as u32,
+                    value_size: 3,
+                    max_entries: 1,
+                    map_flags: BPF_F_RDONLY_PROG,
+                    id: 1,
+                    pinning: PinningType::None,
+                },
+                section_index: 1,
+                data: vec![0, 0, 0],
+                kind: MapKind::Rodata,
+            },
+        );
+        obj.symbols_by_index.insert(
+            1,
+            Symbol {
+                index: 1,
+                section_index: Some(1),
+                name: Some("my_config".to_string()),
+                address: 0,
+                size: 3,
+                is_definition: true,
+                kind: SymbolKind::Data,
+            },
+        );
+
+        let test_data: &[u8] = &[1, 2, 3];
+        obj.patch_map_data(HashMap::from([("my_config", test_data)]))
+            .unwrap();
+
+        let map = obj.maps.get(".rodata").unwrap();
+        assert_eq!(test_data, map.data);
+    }
 }
