@@ -19,7 +19,7 @@
 //!
 //! let mut bpf = Bpf::load_file("ebpf_programs.o")?;
 //! // intercept_wakeups is the name of the program we want to load
-//! let program: &mut KProbe = bpf.program_mut("intercept_wakeups")?.try_into()?;
+//! let program: &mut KProbe = bpf.program_mut("intercept_wakeups").unwrap().try_into()?;
 //! program.load()?;
 //! // intercept_wakeups will be called every time try_to_wake_up() is called
 //! // inside the kernel
@@ -37,26 +37,32 @@
 //! [`Bpf::program_mut`]: crate::Bpf::program_mut
 //! [`maps`]: crate::maps
 mod cgroup_skb;
+mod extension;
+mod fentry;
+mod fexit;
 mod kprobe;
 mod lirc_mode2;
+mod lsm;
 mod perf_attach;
 pub mod perf_event;
 mod probe;
+mod raw_trace_point;
 mod sk_msg;
 mod sk_skb;
 mod sock_ops;
 mod socket_filter;
 pub mod tc;
+mod tp_btf;
 mod trace_point;
 mod uprobe;
+mod utils;
 mod xdp;
 
 use libc::{close, dup, ENOSPC};
 use std::{
     cell::RefCell,
-    cmp,
     convert::TryFrom,
-    ffi::{CStr, CString},
+    ffi::CString,
     io,
     os::unix::io::{AsRawFd, RawFd},
     path::Path,
@@ -65,16 +71,22 @@ use std::{
 use thiserror::Error;
 
 pub use cgroup_skb::{CgroupSkb, CgroupSkbAttachType};
+pub use extension::{Extension, ExtensionError};
+pub use fentry::FEntry;
+pub use fexit::FExit;
 pub use kprobe::{KProbe, KProbeError};
 pub use lirc_mode2::LircMode2;
+pub use lsm::Lsm;
 use perf_attach::*;
 pub use perf_event::{PerfEvent, PerfEventScope, PerfTypeId, SamplePolicy};
 pub use probe::ProbeKind;
+pub use raw_trace_point::RawTracePoint;
 pub use sk_msg::SkMsg;
 pub use sk_skb::{SkSkb, SkSkbKind};
 pub use sock_ops::SockOps;
 pub use socket_filter::{SocketFilter, SocketFilterError};
 pub use tc::{SchedClassifier, TcAttachType, TcError};
+pub use tp_btf::BtfTracePoint;
 pub use trace_point::{TracePoint, TracePointError};
 pub use uprobe::{UProbe, UProbeError};
 pub use xdp::{Xdp, XdpError, XdpFlags};
@@ -82,17 +94,17 @@ pub use xdp::{Xdp, XdpError, XdpFlags};
 use crate::{
     generated::{bpf_attach_type, bpf_prog_info, bpf_prog_type},
     maps::MapError,
-    obj::{self, Function},
-    sys::{bpf_load_program, bpf_pin_object, bpf_prog_detach, bpf_prog_query},
+    obj::{self, btf::BtfError, Function, KernelVersion},
+    sys::{
+        bpf_get_object, bpf_load_program, bpf_obj_get_info_by_fd, bpf_pin_object, bpf_prog_detach,
+        bpf_prog_get_fd_by_id, bpf_prog_query, retry_with_verifier_logs, BpfLoadProgramAttrs,
+    },
+    util::VerifierLog,
 };
 
 /// Error type returned when working with programs.
 #[derive(Debug, Error)]
 pub enum ProgramError {
-    /// The program could not be found in the object code.
-    #[error("program `{name}` not found")]
-    NotFound { name: String },
-
     /// The program is already loaded.
     #[error("the program is already loaded")]
     AlreadyLoaded,
@@ -131,14 +143,21 @@ pub enum ProgramError {
 
     /// The network interface does not exist.
     #[error("unknown network interface {name}")]
-    UnknownInterface { name: String },
+    UnknownInterface {
+        /// interface name
+        name: String,
+    },
 
     /// The program is not of the expected type.
     #[error("unexpected program type")]
     UnexpectedProgramType,
 
+    /// Invalid pin path
     #[error("invalid pin path `{error}`")]
-    InvalidPinPath { error: String },
+    InvalidPinPath {
+        /// the error message
+        error: String,
+    },
 
     /// A map error occurred while loading or attaching a program.
     #[error(transparent)]
@@ -167,27 +186,68 @@ pub enum ProgramError {
     /// An error occurred while working with a TC program.
     #[error(transparent)]
     TcError(#[from] TcError),
+
+    /// An error occurred while working with an [`Extension`] program.
+    #[error(transparent)]
+    ExtensionError(#[from] ExtensionError),
+
+    /// An error occurred while working with BTF.
+    #[error(transparent)]
+    Btf(#[from] BtfError),
+
+    /// The program is not attached.
+    #[error("the program name `{name}` is invalid")]
+    InvalidName {
+        /// program name
+        name: String,
+    },
 }
 
+/// Allows the Fd of a loaded [`Program`] to be retrieved
 pub trait ProgramFd {
+    /// Returns the [`RawFd`] of the program if it has been loaded, or `None`
     fn fd(&self) -> Option<RawFd>;
 }
 
 /// eBPF program type.
 #[derive(Debug)]
 pub enum Program {
+    /// A [`KProbe`] program
     KProbe(KProbe),
+    /// A [`UProbe`] program
     UProbe(UProbe),
+    /// A [`TracePoint`] program
     TracePoint(TracePoint),
+    /// A [`SocketFilter`] program
     SocketFilter(SocketFilter),
+    /// A [`Xdp`] program
     Xdp(Xdp),
+    /// A [`SkMsg`] program
     SkMsg(SkMsg),
+    /// A [`SkSkb`] program
     SkSkb(SkSkb),
+    /// A [`SockOps`] program
     SockOps(SockOps),
+    /// A [`SchedClassifier`] program
     SchedClassifier(SchedClassifier),
+    /// A [`CgroupSkb`] program
     CgroupSkb(CgroupSkb),
+    /// A [`LircMode2`] program
     LircMode2(LircMode2),
+    /// A [`PerfEvent`] program
     PerfEvent(PerfEvent),
+    /// A [`RawTracePoint`] program
+    RawTracePoint(RawTracePoint),
+    /// A [`Lsm`] program
+    Lsm(Lsm),
+    /// A [`BtfTracePoint`] program
+    BtfTracePoint(BtfTracePoint),
+    /// A [`FEntry`] program
+    FEntry(FEntry),
+    /// A [`FExit`] program
+    FExit(FExit),
+    /// A [`Extension`] program
+    Extension(Extension),
 }
 
 impl Program {
@@ -221,14 +281,15 @@ impl Program {
             Program::CgroupSkb(_) => BPF_PROG_TYPE_CGROUP_SKB,
             Program::LircMode2(_) => BPF_PROG_TYPE_LIRC_MODE2,
             Program::PerfEvent(_) => BPF_PROG_TYPE_PERF_EVENT,
+            Program::RawTracePoint(_) => BPF_PROG_TYPE_RAW_TRACEPOINT,
+            Program::Lsm(_) => BPF_PROG_TYPE_LSM,
+            Program::BtfTracePoint(_) => BPF_PROG_TYPE_TRACING,
+            Program::FEntry(_) => BPF_PROG_TYPE_TRACING,
+            Program::FExit(_) => BPF_PROG_TYPE_TRACING,
+            Program::Extension(_) => BPF_PROG_TYPE_EXT,
         }
     }
 
-    /// Returns the name of the program.
-    pub fn name(&self) -> &str {
-        &self.data().name
-    }
-
     /// Pin the program to the provided path
     pub fn pin<P: AsRef<Path>>(&mut self, path: P) -> Result<(), ProgramError> {
         self.data_mut().pin(path)
@@ -248,6 +309,12 @@ impl Program {
             Program::CgroupSkb(p) => &p.data,
             Program::LircMode2(p) => &p.data,
             Program::PerfEvent(p) => &p.data,
+            Program::RawTracePoint(p) => &p.data,
+            Program::Lsm(p) => &p.data,
+            Program::BtfTracePoint(p) => &p.data,
+            Program::FEntry(p) => &p.data,
+            Program::FExit(p) => &p.data,
+            Program::Extension(p) => &p.data,
         }
     }
 
@@ -265,16 +332,27 @@ impl Program {
             Program::CgroupSkb(p) => &mut p.data,
             Program::LircMode2(p) => &mut p.data,
             Program::PerfEvent(p) => &mut p.data,
+            Program::RawTracePoint(p) => &mut p.data,
+            Program::Lsm(p) => &mut p.data,
+            Program::BtfTracePoint(p) => &mut p.data,
+            Program::FEntry(p) => &mut p.data,
+            Program::FExit(p) => &mut p.data,
+            Program::Extension(p) => &mut p.data,
         }
     }
 }
 
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub(crate) struct ProgramData {
-    pub(crate) name: String,
+    pub(crate) name: Option<String>,
     pub(crate) obj: obj::Program,
     pub(crate) fd: Option<RawFd>,
     pub(crate) links: Vec<Rc<RefCell<dyn Link>>>,
+    pub(crate) expected_attach_type: Option<bpf_attach_type>,
+    pub(crate) attach_btf_obj_fd: Option<u32>,
+    pub(crate) attach_btf_id: Option<u32>,
+    pub(crate) attach_prog_fd: Option<RawFd>,
+    pub(crate) btf_fd: Option<RawFd>,
 }
 
 impl ProgramData {
@@ -306,111 +384,82 @@ impl ProgramData {
     }
 }
 
-const MIN_LOG_BUF_SIZE: usize = 1024 * 10;
-const MAX_LOG_BUF_SIZE: usize = (std::u32::MAX >> 8) as usize;
-
-pub(crate) struct VerifierLog {
-    buf: Vec<u8>,
-}
-
-impl VerifierLog {
-    fn new() -> VerifierLog {
-        VerifierLog { buf: Vec::new() }
-    }
-
-    pub(crate) fn buf(&mut self) -> &mut Vec<u8> {
-        &mut self.buf
-    }
-
-    fn grow(&mut self) {
-        let len = cmp::max(
-            MIN_LOG_BUF_SIZE,
-            cmp::min(MAX_LOG_BUF_SIZE, self.buf.capacity() * 10),
-        );
-        self.buf.resize(len, 0);
-        self.reset();
-    }
-
-    fn reset(&mut self) {
-        if !self.buf.is_empty() {
-            self.buf[0] = 0;
-        }
-    }
-
-    fn truncate(&mut self) {
-        if self.buf.is_empty() {
-            return;
-        }
-
-        let pos = self
-            .buf
-            .iter()
-            .position(|b| *b == 0)
-            .unwrap_or(self.buf.len() - 1);
-        self.buf[pos] = 0;
-        self.buf.truncate(pos + 1);
-    }
-
-    pub fn as_c_str(&self) -> Option<&CStr> {
-        if self.buf.is_empty() {
-            None
-        } else {
-            Some(CStr::from_bytes_with_nul(&self.buf).unwrap())
-        }
-    }
-}
-
 fn load_program(prog_type: bpf_prog_type, data: &mut ProgramData) -> Result<(), ProgramError> {
     let ProgramData { obj, fd, .. } = data;
     if fd.is_some() {
         return Err(ProgramError::AlreadyLoaded);
     }
     let crate::obj::Program {
-        function: Function { instructions, .. },
+        function:
+            Function {
+                instructions,
+                func_info,
+                line_info,
+                func_info_rec_size,
+                line_info_rec_size,
+                ..
+            },
         license,
         kernel_version,
         ..
     } = obj;
 
-    let mut log_buf = VerifierLog::new();
-    let mut retries = 0;
-    let mut ret;
-    loop {
-        ret = bpf_load_program(
-            prog_type,
-            instructions,
-            license,
-            (*kernel_version).into(),
-            &mut log_buf,
-        );
-        match &ret {
-            Ok(prog_fd) => {
-                *fd = Some(*prog_fd as RawFd);
-                return Ok(());
-            }
-            Err((_, io_error)) if retries == 0 || io_error.raw_os_error() == Some(ENOSPC) => {
-                if retries == 10 {
-                    break;
-                }
-                retries += 1;
-                log_buf.grow();
-            }
-            Err(_) => break,
-        };
-    }
+    let target_kernel_version = match *kernel_version {
+        KernelVersion::Any => {
+            let (major, minor, patch) = crate::sys::kernel_version().unwrap();
+            (major << 16) + (minor << 8) + patch
+        }
+        _ => (*kernel_version).into(),
+    };
 
-    if let Err((_, io_error)) = ret {
-        log_buf.truncate();
-        return Err(ProgramError::LoadError {
-            io_error,
-            verifier_log: log_buf
-                .as_c_str()
-                .map(|s| s.to_string_lossy().to_string())
-                .unwrap_or_else(|| "[none]".to_owned()),
-        });
-    }
+    let mut logger = VerifierLog::new();
 
-    Ok(())
+    let prog_name = if let Some(name) = &data.name {
+        let mut name = name.clone();
+        if name.len() > 15 {
+            name.truncate(15);
+        }
+        let prog_name = CString::new(name.clone())
+            .map_err(|_| ProgramError::InvalidName { name: name.clone() })?;
+        Some(prog_name)
+    } else {
+        None
+    };
+
+    let attr = BpfLoadProgramAttrs {
+        name: prog_name,
+        ty: prog_type,
+        insns: instructions,
+        license,
+        kernel_version: target_kernel_version,
+        expected_attach_type: data.expected_attach_type,
+        prog_btf_fd: data.btf_fd,
+        attach_btf_obj_fd: data.attach_btf_obj_fd,
+        attach_btf_id: data.attach_btf_id,
+        attach_prog_fd: data.attach_prog_fd,
+        func_info_rec_size: *func_info_rec_size,
+        func_info: func_info.clone(),
+        line_info_rec_size: *line_info_rec_size,
+        line_info: line_info.clone(),
+    };
+    let ret = retry_with_verifier_logs(10, &mut logger, |logger| bpf_load_program(&attr, logger));
+
+    match ret {
+        Ok(prog_fd) => {
+            *fd = Some(prog_fd as RawFd);
+            Ok(())
+        }
+        Err((_, io_error)) => {
+            logger.truncate();
+            return Err(ProgramError::LoadError {
+                io_error,
+                verifier_log: logger
+                    .as_c_str()
+                    .map(|s| s.to_string_lossy().to_string())
+                    .unwrap_or_else(|| "[none]".to_owned()),
+            });
+        }
+    }
 }
 
 pub(crate) fn query<T: AsRawFd>(
@@ -451,8 +500,9 @@ pub(crate) fn query<T: AsRawFd>(
     }
 }
 
-/// Detach an attached program.
+/// Detach an attached program
 pub trait Link: std::fmt::Debug {
+    /// detaches an attached program
     fn detach(&mut self) -> Result<(), ProgramError>;
 }
 
@@ -545,6 +595,12 @@ impl ProgramFd for Program {
     }
 }
 
+impl<'a, P: ProgramFd> ProgramFd for &'a P {
+    fn fd(&self) -> Option<RawFd> {
+        (*self).fd()
+    }
+}
+
 macro_rules! impl_program_fd {
     ($($struct_name:ident),+ $(,)?) => {
         $(
@@ -553,6 +609,12 @@ macro_rules! impl_program_fd {
                     self.data.fd
                 }
             }
+
+            impl ProgramFd for &mut $struct_name {
+                fn fd(&self) -> Option<RawFd> {
+                    self.data.fd
+                }
+            }
         )+
     }
 }
@@ -568,7 +630,13 @@ impl_program_fd!(
     SchedClassifier,
     CgroupSkb,
     LircMode2,
-    PerfEvent
+    PerfEvent,
+    Lsm,
+    RawTracePoint,
+    BtfTracePoint,
+    FEntry,
+    FExit,
+    Extension,
 );
 
 macro_rules! impl_try_from_program {
@@ -611,7 +679,13 @@ impl_try_from_program!(
     SchedClassifier,
     CgroupSkb,
     LircMode2,
-    PerfEvent
+    PerfEvent,
+    Lsm,
+    RawTracePoint,
+    BtfTracePoint,
+    FEntry,
+    FExit,
+    Extension,
 );
 
 /// Provides information about a loaded program, like name, id and statistics
@@ -642,4 +716,44 @@ impl ProgramInfo {
     pub fn id(&self) -> u32 {
         self.0.id
     }
+
+    /// Returns the fd associated with the program.
+    ///
+    /// The returned fd must be closed when no longer needed.
+    pub fn fd(&self) -> Result<RawFd, ProgramError> {
+        let fd =
+            bpf_prog_get_fd_by_id(self.0.id).map_err(|io_error| ProgramError::SyscallError {
+                call: "bpf_prog_get_fd_by_id".to_owned(),
+                io_error,
+            })?;
+        Ok(fd as RawFd)
+    }
+
+    /// Loads a program from a pinned path in bpffs.
+    pub fn from_pinned<P: AsRef<Path>>(path: P) -> Result<ProgramInfo, ProgramError> {
+        let path_string = match CString::new(path.as_ref().to_str().unwrap()) {
+            Ok(s) => s,
+            Err(e) => {
+                return Err(ProgramError::InvalidPinPath {
+                    error: e.to_string(),
+                })
+            }
+        };
+        let fd =
+            bpf_get_object(&path_string).map_err(|(_, io_error)| ProgramError::SyscallError {
+                call: "bpf_obj_get".to_owned(),
+                io_error,
+            })? as RawFd;
+
+        let info = bpf_obj_get_info_by_fd(fd).map_err(|io_error| ProgramError::SyscallError {
+            call: "bpf_obj_get_info_by_fd".to_owned(),
+            io_error,
+        })?;
+
+        unsafe {
+            libc::close(fd);
+        }
+
+        Ok(ProgramInfo(info))
+    }
 }
