@@ -1,15 +1,17 @@
 //! Utility functions.
 use std::{
+    cmp,
     collections::BTreeMap,
-    ffi::CString,
+    ffi::{CStr, CString},
     fs::{self, File},
     io::{self, BufReader},
+    mem, slice,
     str::FromStr,
 };
 
 use crate::generated::{TC_H_MAJ_MASK, TC_H_MIN_MASK};
 
-use libc::if_nametoindex;
+use libc::{if_nametoindex, sysconf, _SC_PAGESIZE};
 
 use io::BufRead;
 
@@ -76,7 +78,7 @@ pub fn kernel_symbols() -> Result<BTreeMap<u64, String>, io::Error> {
     parse_kernel_symbols(&mut reader)
 }
 
-fn parse_kernel_symbols(reader: &mut dyn BufRead) -> Result<BTreeMap<u64, String>, io::Error> {
+fn parse_kernel_symbols(reader: impl BufRead) -> Result<BTreeMap<u64, String>, io::Error> {
     let mut syms = BTreeMap::new();
 
     for line in reader.lines() {
@@ -106,6 +108,108 @@ pub(crate) fn tc_handler_make(major: u32, minor: u32) -> u32 {
     (major & TC_H_MAJ_MASK) | (minor & TC_H_MIN_MASK)
 }
 
+/// Include bytes from a file for use in a subsequent [`crate::Bpf::load`].
+///
+/// This macro differs from the standard `include_bytes!` macro since it also ensures that
+/// the bytes are correctly aligned to be parsed as an ELF binary. This avoid some nasty
+/// compilation errors when the resulting byte array is not the correct alignment.
+///
+/// # Examples
+/// ```ignore
+/// use aya::{Bpf, include_bytes_aligned};
+///
+/// let mut bpf = Bpf::load(include_bytes_aligned!(
+///     "/path/to/bpf.o"
+/// ))?;
+///
+/// # Ok::<(), aya::BpfError>(())
+/// ```
+#[macro_export]
+macro_rules! include_bytes_aligned {
+    ($path:expr) => {{
+        #[repr(align(32))]
+        pub struct Aligned32;
+
+        #[repr(C)]
+        pub struct Aligned<Bytes: ?Sized> {
+            pub _align: [Aligned32; 0],
+            pub bytes: Bytes,
+        }
+
+        static ALIGNED: &Aligned<[u8]> = &Aligned {
+            _align: [],
+            bytes: *include_bytes!($path),
+        };
+
+        &ALIGNED.bytes
+    }};
+}
+
+pub(crate) fn page_size() -> usize {
+    // Safety: libc
+    (unsafe { sysconf(_SC_PAGESIZE) }) as usize
+}
+
+// bytes_of converts a <T> to a byte slice
+pub(crate) unsafe fn bytes_of<T>(val: &T) -> &[u8] {
+    let size = mem::size_of::<T>();
+    slice::from_raw_parts(slice::from_ref(val).as_ptr().cast(), size)
+}
+
+const MIN_LOG_BUF_SIZE: usize = 1024 * 10;
+const MAX_LOG_BUF_SIZE: usize = (std::u32::MAX >> 8) as usize;
+
+pub(crate) struct VerifierLog {
+    buf: Vec<u8>,
+}
+
+impl VerifierLog {
+    pub(crate) fn new() -> VerifierLog {
+        VerifierLog { buf: Vec::new() }
+    }
+
+    pub(crate) fn buf(&mut self) -> &mut Vec<u8> {
+        &mut self.buf
+    }
+
+    pub(crate) fn grow(&mut self) {
+        let len = cmp::max(
+            MIN_LOG_BUF_SIZE,
+            cmp::min(MAX_LOG_BUF_SIZE, self.buf.capacity() * 10),
+        );
+        self.buf.resize(len, 0);
+        self.reset();
+    }
+
+    pub(crate) fn reset(&mut self) {
+        if !self.buf.is_empty() {
+            self.buf[0] = 0;
+        }
+    }
+
+    pub(crate) fn truncate(&mut self) {
+        if self.buf.is_empty() {
+            return;
+        }
+
+        let pos = self
+            .buf
+            .iter()
+            .position(|b| *b == 0)
+            .unwrap_or(self.buf.len() - 1);
+        self.buf[pos] = 0;
+        self.buf.truncate(pos + 1);
+    }
+
+    pub(crate) fn as_c_str(&self) -> Option<&CStr> {
+        if self.buf.is_empty() {
+            None
+        } else {
+            Some(CStr::from_bytes_with_nul(&self.buf).unwrap())
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
